# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: mqtt_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'mqtt_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12mqtt_service.proto\x12\x08isa.mqtt\x1a\x0c\x63ommon.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto\"\xb1\x03\n\nDeviceInfo\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65vice_name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65vice_type\x18\x03 \x01(\t\x12\x0f\n\x07user_id\x18\x04 \x01(\t\x12\x17\n\x0forganization_id\x18\x05 \x01(\t\x12&\n\x06status\x18\x06 \x01(\x0e\x32\x16.isa.mqtt.DeviceStatus\x12\x31\n\rregistered_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\tlast_seen\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x34\n\x08metadata\x18\t \x03(\x0b\x32\".isa.mqtt.DeviceInfo.MetadataEntry\x12\x19\n\x11subscribed_topics\x18\n \x03(\t\x12\x15\n\rmessages_sent\x18\x0b \x01(\x03\x12\x19\n\x11messages_received\x18\x0c \x01(\x03\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xca\x01\n\tTopicInfo\x12\r\n\x05topic\x18\x01 \x01(\t\x12\x18\n\x10subscriber_count\x18\x02 \x01(\x03\x12\x15\n\rmessage_count\x18\x03 \x01(\x03\x12\x35\n\x11last_message_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1c\n\x14has_retained_message\x18\x05 \x01(\x08\x12\x0f\n\x07user_id\x18\x06 \x01(\t\x12\x17\n\x0forganization_id\x18\x07 \x01(\t\"\x91\x02\n\x0bMQTTMessage\x12\r\n\x05topic\x18\x01 \x01(\t\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\x12\x1f\n\x03qos\x18\x03 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\x12\x10\n\x08retained\x18\x04 \x01(\x08\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nmessage_id\x18\x06 \x01(\t\x12\x39\n\nproperties\x18\x07 \x03(\x0b\x32%.isa.mqtt.MQTTMessage.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xe8\x01\n\x0e\x43onnectRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tclient_id\x18\x02 \x01(\t\x12\x10\n\x08username\x18\x03 \x01(\t\x12\x10\n\x08password\x18\x04 \x01(\t\x12\x15\n\rclean_session\x18\x05 \x01(\x08\x12\x12\n\nkeep_alive\x18\x06 \x01(\x05\x12\x12\n\nwill_topic\x18\x07 \x01(\t\x12\x14\n\x0cwill_payload\x18\x08 \x01(\x0c\x12$\n\x08will_qos\x18\t \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\x12\x13\n\x0bwill_retain\x18\n \x01(\x08\"`\n\x0f\x43onnectResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x17\n\x0fsession_present\x18\x04 \x01(\x08\"8\n\x11\x44isconnectRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\"6\n\x12\x44isconnectResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\">\n\x17\x43onnectionStatusRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\"\x91\x01\n\x18\x43onnectionStatusResponse\x12\x11\n\tconnected\x18\x01 \x01(\x08\x12\x30\n\x0c\x63onnected_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rmessages_sent\x18\x03 \x01(\x03\x12\x19\n\x11messages_received\x18\x04 \x01(\x03\"\xf9\x01\n\x0ePublishRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\r\n\x05topic\x18\x03 \x01(\t\x12\x0f\n\x07payload\x18\x04 \x01(\x0c\x12\x1f\n\x03qos\x18\x05 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\x12\x10\n\x08retained\x18\x06 \x01(\x08\x12<\n\nproperties\x18\x07 \x03(\x0b\x32(.isa.mqtt.PublishRequest.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"y\n\x0fPublishResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x12\n\nmessage_id\x18\x02 \x01(\t\x12\x0f\n\x07message\x18\x03 \x01(\t\x12\x30\n\x0cpublished_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"f\n\x13PublishBatchRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12*\n\x08messages\x18\x03 \x03(\x0b\x32\x18.isa.mqtt.PublishRequest\"{\n\x14PublishBatchResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x17\n\x0fpublished_count\x18\x02 \x01(\x05\x12\x14\n\x0c\x66\x61iled_count\x18\x03 \x01(\x05\x12\x13\n\x0bmessage_ids\x18\x04 \x03(\t\x12\x0e\n\x06\x65rrors\x18\x05 \x03(\t\"\xa2\x01\n\x12PublishJSONRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\r\n\x05topic\x18\x03 \x01(\t\x12%\n\x04\x64\x61ta\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x1f\n\x03qos\x18\x05 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\x12\x10\n\x08retained\x18\x06 \x01(\x08\"n\n\x10SubscribeRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x14\n\x0ctopic_filter\x18\x03 \x01(\t\x12\x1f\n\x03qos\x18\x04 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\"s\n\x18SubscribeMultipleRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x32\n\rsubscriptions\x18\x03 \x03(\x0b\x32\x1b.isa.mqtt.TopicSubscription\"J\n\x11TopicSubscription\x12\x14\n\x0ctopic_filter\x18\x01 \x01(\t\x12\x1f\n\x03qos\x18\x02 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\"\x99\x02\n\x0fMessageResponse\x12\r\n\x05topic\x18\x01 \x01(\t\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\x12\x1f\n\x03qos\x18\x03 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\x12\x10\n\x08retained\x18\x04 \x01(\x08\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\nmessage_id\x18\x06 \x01(\t\x12=\n\nproperties\x18\x07 \x03(\x0b\x32).isa.mqtt.MessageResponse.PropertiesEntry\x1a\x31\n\x0fPropertiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"P\n\x12UnsubscribeRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\x12\x15\n\rtopic_filters\x18\x03 \x03(\t\"S\n\x13UnsubscribeResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x1a\n\x12unsubscribed_count\x18\x02 \x01(\x05\x12\x0f\n\x07message\x18\x03 \x01(\t\"?\n\x18ListSubscriptionsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nsession_id\x18\x02 \x01(\t\"O\n\x19ListSubscriptionsResponse\x12\x32\n\rsubscriptions\x18\x01 \x03(\x0b\x32\x1b.isa.mqtt.TopicSubscription\"\xf0\x01\n\x15RegisterDeviceRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x11\n\tdevice_id\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65vice_name\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65vice_type\x18\x05 \x01(\t\x12?\n\x08metadata\x18\x06 \x03(\x0b\x32-.isa.mqtt.RegisterDeviceRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"`\n\x16RegisterDeviceResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12$\n\x06\x64\x65vice\x18\x02 \x01(\x0b\x32\x14.isa.mqtt.DeviceInfo\x12\x0f\n\x07message\x18\x03 \x01(\t\"=\n\x17UnregisterDeviceRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tdevice_id\x18\x02 \x01(\t\"<\n\x18UnregisterDeviceResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"\x87\x01\n\x12ListDevicesRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12&\n\x06status\x18\x03 \x01(\x0e\x32\x16.isa.mqtt.DeviceStatus\x12\x0c\n\x04page\x18\x04 \x01(\x05\x12\x11\n\tpage_size\x18\x05 \x01(\x05\"r\n\x13ListDevicesResponse\x12%\n\x07\x64\x65vices\x18\x01 \x03(\x0b\x32\x14.isa.mqtt.DeviceInfo\x12\x13\n\x0btotal_count\x18\x02 \x01(\x05\x12\x0c\n\x04page\x18\x03 \x01(\x05\x12\x11\n\tpage_size\x18\x04 \x01(\x05\":\n\x14GetDeviceInfoRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tdevice_id\x18\x02 \x01(\t\"=\n\x15GetDeviceInfoResponse\x12$\n\x06\x64\x65vice\x18\x01 \x01(\x0b\x32\x14.isa.mqtt.DeviceInfo\"\xdd\x01\n\x19UpdateDeviceStatusRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tdevice_id\x18\x02 \x01(\t\x12&\n\x06status\x18\x03 \x01(\x0e\x32\x16.isa.mqtt.DeviceStatus\x12\x43\n\x08metadata\x18\x04 \x03(\x0b\x32\x31.isa.mqtt.UpdateDeviceStatusRequest.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"S\n\x1aUpdateDeviceStatusResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12$\n\x06\x64\x65vice\x18\x02 \x01(\x0b\x32\x14.isa.mqtt.DeviceInfo\"5\n\x13GetTopicInfoRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\"?\n\x14GetTopicInfoResponse\x12\'\n\ntopic_info\x18\x01 \x01(\x0b\x32\x13.isa.mqtt.TopicInfo\"n\n\x11ListTopicsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0e\n\x06prefix\x18\x03 \x01(\t\x12\x0c\n\x04page\x18\x04 \x01(\x05\x12\x11\n\tpage_size\x18\x05 \x01(\x05\"N\n\x12ListTopicsResponse\x12#\n\x06topics\x18\x01 \x03(\x0b\x32\x13.isa.mqtt.TopicInfo\x12\x13\n\x0btotal_count\x18\x02 \x01(\x05\">\n\x14ValidateTopicRequest\x12\r\n\x05topic\x18\x01 \x01(\t\x12\x17\n\x0f\x61llow_wildcards\x18\x02 \x01(\x08\"K\n\x15ValidateTopicResponse\x12\r\n\x05valid\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x12\n\nviolations\x18\x03 \x03(\t\"m\n\x19SetRetainedMessageRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\x12\x0f\n\x07payload\x18\x03 \x01(\x0c\x12\x1f\n\x03qos\x18\x04 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\">\n\x1aSetRetainedMessageResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\";\n\x19GetRetainedMessageRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\"S\n\x1aGetRetainedMessageResponse\x12\r\n\x05\x66ound\x18\x01 \x01(\x08\x12&\n\x07message\x18\x02 \x01(\x0b\x32\x15.isa.mqtt.MQTTMessage\">\n\x1c\x44\x65leteRetainedMessageRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\r\n\x05topic\x18\x02 \x01(\t\"A\n\x1d\x44\x65leteRetainedMessageResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"@\n\x14GetStatisticsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\"\xee\x02\n\x15GetStatisticsResponse\x12\x15\n\rtotal_devices\x18\x01 \x01(\x03\x12\x16\n\x0eonline_devices\x18\x02 \x01(\x03\x12\x14\n\x0ctotal_topics\x18\x03 \x01(\x03\x12\x1b\n\x13total_subscriptions\x18\x04 \x01(\x03\x12\x1b\n\x13messages_sent_today\x18\x05 \x01(\x03\x12\x1f\n\x17messages_received_today\x18\x06 \x01(\x03\x12\x17\n\x0f\x61\x63tive_sessions\x18\x07 \x01(\x03\x12]\n\x18\x64\x65vice_type_distribution\x18\x08 \x03(\x0b\x32;.isa.mqtt.GetStatisticsResponse.DeviceTypeDistributionEntry\x1a=\n\x1b\x44\x65viceTypeDistributionEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x03:\x02\x38\x01\"\x9b\x01\n\x17GetDeviceMetricsRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x11\n\tdevice_id\x18\x02 \x01(\t\x12.\n\nstart_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xeb\x01\n\x18GetDeviceMetricsResponse\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12\x15\n\rmessages_sent\x18\x02 \x01(\x03\x12\x19\n\x11messages_received\x18\x03 \x01(\x03\x12\x12\n\nbytes_sent\x18\x04 \x01(\x03\x12\x16\n\x0e\x62ytes_received\x18\x05 \x01(\x03\x12/\n\x0cmessage_rate\x18\x06 \x03(\x0b\x32\x19.isa.mqtt.TimeSeriesPoint\x12-\n\nerror_rate\x18\x07 \x03(\x0b\x32\x19.isa.mqtt.TimeSeriesPoint\"O\n\x0fTimeSeriesPoint\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05value\x18\x02 \x01(\x01\",\n\x16MQTTHealthCheckRequest\x12\x12\n\ndeep_check\x18\x01 \x01(\x08\"\x9e\x01\n\x17MQTTHealthCheckResponse\x12\x0f\n\x07healthy\x18\x01 \x01(\x08\x12\x15\n\rbroker_status\x18\x02 \x01(\t\x12\x1a\n\x12\x61\x63tive_connections\x18\x03 \x01(\x03\x12.\n\nchecked_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07message\x18\x05 \x01(\t\"\xaa\x01\n\x1eSubscribeDeviceMessagesRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x32\n\rmessage_types\x18\x03 \x03(\x0e\x32\x1b.isa.mqtt.DeviceMessageType\x12\x12\n\ndevice_ids\x18\x04 \x03(\t\x12\x16\n\x0etopic_patterns\x18\x05 \x03(\t\"\xaf\x02\n\rDeviceMessage\x12\x11\n\tdevice_id\x18\x01 \x01(\t\x12\x31\n\x0cmessage_type\x18\x02 \x01(\x0e\x32\x1b.isa.mqtt.DeviceMessageType\x12\r\n\x05topic\x18\x03 \x01(\t\x12\x0f\n\x07payload\x18\x04 \x01(\x0c\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x37\n\x08metadata\x18\x06 \x03(\x0b\x32%.isa.mqtt.DeviceMessage.MetadataEntry\x12\x1f\n\x03qos\x18\x07 \x01(\x0e\x32\x12.isa.mqtt.QoSLevel\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa3\x03\n\x0bWebhookInfo\x12\x12\n\nwebhook_id\x18\x01 \x01(\t\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\t\x12\x32\n\rmessage_types\x18\x04 \x03(\x0e\x32\x1b.isa.mqtt.DeviceMessageType\x12\x12\n\ndevice_ids\x18\x05 \x03(\t\x12\x16\n\x0etopic_patterns\x18\x06 \x03(\t\x12\x33\n\x07headers\x18\x07 \x03(\x0b\x32\".isa.mqtt.WebhookInfo.HeadersEntry\x12\x0f\n\x07\x65nabled\x18\x08 \x01(\x08\x12.\n\ncreated_at\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nupdated_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x15\n\rsuccess_count\x18\x0b \x01(\x03\x12\x15\n\rfailure_count\x18\x0c \x01(\x03\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xaf\x02\n\x16RegisterWebhookRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x0b\n\x03url\x18\x03 \x01(\t\x12\x32\n\rmessage_types\x18\x04 \x03(\x0e\x32\x1b.isa.mqtt.DeviceMessageType\x12\x12\n\ndevice_ids\x18\x05 \x03(\t\x12\x16\n\x0etopic_patterns\x18\x06 \x03(\t\x12>\n\x07headers\x18\x07 \x03(\x0b\x32-.isa.mqtt.RegisterWebhookRequest.HeadersEntry\x12\x0e\n\x06secret\x18\x08 \x01(\t\x1a.\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"w\n\x17RegisterWebhookResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x12\n\nwebhook_id\x18\x02 \x01(\t\x12&\n\x07webhook\x18\x03 \x01(\x0b\x32\x15.isa.mqtt.WebhookInfo\x12\x0f\n\x07message\x18\x04 \x01(\t\"?\n\x18UnregisterWebhookRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x12\n\nwebhook_id\x18\x02 \x01(\t\"=\n\x19UnregisterWebhookResponse\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"Y\n\x13ListWebhooksRequest\x12\x0f\n\x07user_id\x18\x01 \x01(\t\x12\x17\n\x0forganization_id\x18\x02 \x01(\t\x12\x18\n\x10include_disabled\x18\x03 \x01(\x08\"T\n\x14ListWebhooksResponse\x12\'\n\x08webhooks\x18\x01 \x03(\x0b\x32\x15.isa.mqtt.WebhookInfo\x12\x13\n\x0btotal_count\x18\x02 \x01(\x05*M\n\x08QoSLevel\x12\x14\n\x10QOS_AT_MOST_ONCE\x10\x00\x12\x15\n\x11QOS_AT_LEAST_ONCE\x10\x01\x12\x14\n\x10QOS_EXACTLY_ONCE\x10\x02*p\n\x0c\x44\x65viceStatus\x12\x12\n\x0e\x44\x45VICE_UNKNOWN\x10\x00\x12\x11\n\rDEVICE_ONLINE\x10\x01\x12\x12\n\x0e\x44\x45VICE_OFFLINE\x10\x02\x12\x13\n\x0f\x44\x45VICE_SLEEPING\x10\x03\x12\x10\n\x0c\x44\x45VICE_ERROR\x10\x04*\xec\x01\n\x11\x44\x65viceMessageType\x12\x1a\n\x16\x44\x45VICE_MESSAGE_UNKNOWN\x10\x00\x12\x1c\n\x18\x44\x45VICE_MESSAGE_TELEMETRY\x10\x01\x12\x19\n\x15\x44\x45VICE_MESSAGE_STATUS\x10\x02\x12\x17\n\x13\x44\x45VICE_MESSAGE_AUTH\x10\x03\x12\x1f\n\x1b\x44\x45VICE_MESSAGE_REGISTRATION\x10\x04\x12#\n\x1f\x44\x45VICE_MESSAGE_COMMAND_RESPONSE\x10\x05\x12#\n\x1f\x44\x45VICE_MESSAGE_NOTIFICATION_ACK\x10\x06\x32\xbc\x12\n\x0bMQTTService\x12>\n\x07\x43onnect\x12\x18.isa.mqtt.ConnectRequest\x1a\x19.isa.mqtt.ConnectResponse\x12G\n\nDisconnect\x12\x1b.isa.mqtt.DisconnectRequest\x1a\x1c.isa.mqtt.DisconnectResponse\x12\\\n\x13GetConnectionStatus\x12!.isa.mqtt.ConnectionStatusRequest\x1a\".isa.mqtt.ConnectionStatusResponse\x12>\n\x07Publish\x12\x18.isa.mqtt.PublishRequest\x1a\x19.isa.mqtt.PublishResponse\x12M\n\x0cPublishBatch\x12\x1d.isa.mqtt.PublishBatchRequest\x1a\x1e.isa.mqtt.PublishBatchResponse\x12\x46\n\x0bPublishJSON\x12\x1c.isa.mqtt.PublishJSONRequest\x1a\x19.isa.mqtt.PublishResponse\x12\x44\n\tSubscribe\x12\x1a.isa.mqtt.SubscribeRequest\x1a\x19.isa.mqtt.MessageResponse0\x01\x12T\n\x11SubscribeMultiple\x12\".isa.mqtt.SubscribeMultipleRequest\x1a\x19.isa.mqtt.MessageResponse0\x01\x12J\n\x0bUnsubscribe\x12\x1c.isa.mqtt.UnsubscribeRequest\x1a\x1d.isa.mqtt.UnsubscribeResponse\x12\\\n\x11ListSubscriptions\x12\".isa.mqtt.ListSubscriptionsRequest\x1a#.isa.mqtt.ListSubscriptionsResponse\x12S\n\x0eRegisterDevice\x12\x1f.isa.mqtt.RegisterDeviceRequest\x1a .isa.mqtt.RegisterDeviceResponse\x12Y\n\x10UnregisterDevice\x12!.isa.mqtt.UnregisterDeviceRequest\x1a\".isa.mqtt.UnregisterDeviceResponse\x12J\n\x0bListDevices\x12\x1c.isa.mqtt.ListDevicesRequest\x1a\x1d.isa.mqtt.ListDevicesResponse\x12P\n\rGetDeviceInfo\x12\x1e.isa.mqtt.GetDeviceInfoRequest\x1a\x1f.isa.mqtt.GetDeviceInfoResponse\x12_\n\x12UpdateDeviceStatus\x12#.isa.mqtt.UpdateDeviceStatusRequest\x1a$.isa.mqtt.UpdateDeviceStatusResponse\x12M\n\x0cGetTopicInfo\x12\x1d.isa.mqtt.GetTopicInfoRequest\x1a\x1e.isa.mqtt.GetTopicInfoResponse\x12G\n\nListTopics\x12\x1b.isa.mqtt.ListTopicsRequest\x1a\x1c.isa.mqtt.ListTopicsResponse\x12P\n\rValidateTopic\x12\x1e.isa.mqtt.ValidateTopicRequest\x1a\x1f.isa.mqtt.ValidateTopicResponse\x12_\n\x12SetRetainedMessage\x12#.isa.mqtt.SetRetainedMessageRequest\x1a$.isa.mqtt.SetRetainedMessageResponse\x12_\n\x12GetRetainedMessage\x12#.isa.mqtt.GetRetainedMessageRequest\x1a$.isa.mqtt.GetRetainedMessageResponse\x12h\n\x15\x44\x65leteRetainedMessage\x12&.isa.mqtt.DeleteRetainedMessageRequest\x1a\'.isa.mqtt.DeleteRetainedMessageResponse\x12P\n\rGetStatistics\x12\x1e.isa.mqtt.GetStatisticsRequest\x1a\x1f.isa.mqtt.GetStatisticsResponse\x12Y\n\x10GetDeviceMetrics\x12!.isa.mqtt.GetDeviceMetricsRequest\x1a\".isa.mqtt.GetDeviceMetricsResponse\x12R\n\x0bHealthCheck\x12 .isa.mqtt.MQTTHealthCheckRequest\x1a!.isa.mqtt.MQTTHealthCheckResponse\x12^\n\x17SubscribeDeviceMessages\x12(.isa.mqtt.SubscribeDeviceMessagesRequest\x1a\x17.isa.mqtt.DeviceMessage0\x01\x12V\n\x0fRegisterWebhook\x12 .isa.mqtt.RegisterWebhookRequest\x1a!.isa.mqtt.RegisterWebhookResponse\x12\\\n\x11UnregisterWebhook\x12\".isa.mqtt.UnregisterWebhookRequest\x1a#.isa.mqtt.UnregisterWebhookResponse\x12M\n\x0cListWebhooks\x12\x1d.isa.mqtt.ListWebhooksRequest\x1a\x1e.isa.mqtt.ListWebhooksResponseB/Z-github.com/isa-cloud/isa_cloud/api/proto/mqttb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mqtt_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z-github.com/isa-cloud/isa_cloud/api/proto/mqtt'
  _globals['_DEVICEINFO_METADATAENTRY']._loaded_options = None
  _globals['_DEVICEINFO_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_MQTTMESSAGE_PROPERTIESENTRY']._loaded_options = None
  _globals['_MQTTMESSAGE_PROPERTIESENTRY']._serialized_options = b'8\001'
  _globals['_PUBLISHREQUEST_PROPERTIESENTRY']._loaded_options = None
  _globals['_PUBLISHREQUEST_PROPERTIESENTRY']._serialized_options = b'8\001'
  _globals['_MESSAGERESPONSE_PROPERTIESENTRY']._loaded_options = None
  _globals['_MESSAGERESPONSE_PROPERTIESENTRY']._serialized_options = b'8\001'
  _globals['_REGISTERDEVICEREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_REGISTERDEVICEREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_UPDATEDEVICESTATUSREQUEST_METADATAENTRY']._loaded_options = None
  _globals['_UPDATEDEVICESTATUSREQUEST_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_GETSTATISTICSRESPONSE_DEVICETYPEDISTRIBUTIONENTRY']._loaded_options = None
  _globals['_GETSTATISTICSRESPONSE_DEVICETYPEDISTRIBUTIONENTRY']._serialized_options = b'8\001'
  _globals['_DEVICEMESSAGE_METADATAENTRY']._loaded_options = None
  _globals['_DEVICEMESSAGE_METADATAENTRY']._serialized_options = b'8\001'
  _globals['_WEBHOOKINFO_HEADERSENTRY']._loaded_options = None
  _globals['_WEBHOOKINFO_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_REGISTERWEBHOOKREQUEST_HEADERSENTRY']._loaded_options = None
  _globals['_REGISTERWEBHOOKREQUEST_HEADERSENTRY']._serialized_options = b'8\001'
  _globals['_QOSLEVEL']._serialized_start=8165
  _globals['_QOSLEVEL']._serialized_end=8242
  _globals['_DEVICESTATUS']._serialized_start=8244
  _globals['_DEVICESTATUS']._serialized_end=8356
  _globals['_DEVICEMESSAGETYPE']._serialized_start=8359
  _globals['_DEVICEMESSAGETYPE']._serialized_end=8595
  _globals['_DEVICEINFO']._serialized_start=110
  _globals['_DEVICEINFO']._serialized_end=543
  _globals['_DEVICEINFO_METADATAENTRY']._serialized_start=496
  _globals['_DEVICEINFO_METADATAENTRY']._serialized_end=543
  _globals['_TOPICINFO']._serialized_start=546
  _globals['_TOPICINFO']._serialized_end=748
  _globals['_MQTTMESSAGE']._serialized_start=751
  _globals['_MQTTMESSAGE']._serialized_end=1024
  _globals['_MQTTMESSAGE_PROPERTIESENTRY']._serialized_start=975
  _globals['_MQTTMESSAGE_PROPERTIESENTRY']._serialized_end=1024
  _globals['_CONNECTREQUEST']._serialized_start=1027
  _globals['_CONNECTREQUEST']._serialized_end=1259
  _globals['_CONNECTRESPONSE']._serialized_start=1261
  _globals['_CONNECTRESPONSE']._serialized_end=1357
  _globals['_DISCONNECTREQUEST']._serialized_start=1359
  _globals['_DISCONNECTREQUEST']._serialized_end=1415
  _globals['_DISCONNECTRESPONSE']._serialized_start=1417
  _globals['_DISCONNECTRESPONSE']._serialized_end=1471
  _globals['_CONNECTIONSTATUSREQUEST']._serialized_start=1473
  _globals['_CONNECTIONSTATUSREQUEST']._serialized_end=1535
  _globals['_CONNECTIONSTATUSRESPONSE']._serialized_start=1538
  _globals['_CONNECTIONSTATUSRESPONSE']._serialized_end=1683
  _globals['_PUBLISHREQUEST']._serialized_start=1686
  _globals['_PUBLISHREQUEST']._serialized_end=1935
  _globals['_PUBLISHREQUEST_PROPERTIESENTRY']._serialized_start=975
  _globals['_PUBLISHREQUEST_PROPERTIESENTRY']._serialized_end=1024
  _globals['_PUBLISHRESPONSE']._serialized_start=1937
  _globals['_PUBLISHRESPONSE']._serialized_end=2058
  _globals['_PUBLISHBATCHREQUEST']._serialized_start=2060
  _globals['_PUBLISHBATCHREQUEST']._serialized_end=2162
  _globals['_PUBLISHBATCHRESPONSE']._serialized_start=2164
  _globals['_PUBLISHBATCHRESPONSE']._serialized_end=2287
  _globals['_PUBLISHJSONREQUEST']._serialized_start=2290
  _globals['_PUBLISHJSONREQUEST']._serialized_end=2452
  _globals['_SUBSCRIBEREQUEST']._serialized_start=2454
  _globals['_SUBSCRIBEREQUEST']._serialized_end=2564
  _globals['_SUBSCRIBEMULTIPLEREQUEST']._serialized_start=2566
  _globals['_SUBSCRIBEMULTIPLEREQUEST']._serialized_end=2681
  _globals['_TOPICSUBSCRIPTION']._serialized_start=2683
  _globals['_TOPICSUBSCRIPTION']._serialized_end=2757
  _globals['_MESSAGERESPONSE']._serialized_start=2760
  _globals['_MESSAGERESPONSE']._serialized_end=3041
  _globals['_MESSAGERESPONSE_PROPERTIESENTRY']._serialized_start=975
  _globals['_MESSAGERESPONSE_PROPERTIESENTRY']._serialized_end=1024
  _globals['_UNSUBSCRIBEREQUEST']._serialized_start=3043
  _globals['_UNSUBSCRIBEREQUEST']._serialized_end=3123
  _globals['_UNSUBSCRIBERESPONSE']._serialized_start=3125
  _globals['_UNSUBSCRIBERESPONSE']._serialized_end=3208
  _globals['_LISTSUBSCRIPTIONSREQUEST']._serialized_start=3210
  _globals['_LISTSUBSCRIPTIONSREQUEST']._serialized_end=3273
  _globals['_LISTSUBSCRIPTIONSRESPONSE']._serialized_start=3275
  _globals['_LISTSUBSCRIPTIONSRESPONSE']._serialized_end=3354
  _globals['_REGISTERDEVICEREQUEST']._serialized_start=3357
  _globals['_REGISTERDEVICEREQUEST']._serialized_end=3597
  _globals['_REGISTERDEVICEREQUEST_METADATAENTRY']._serialized_start=496
  _globals['_REGISTERDEVICEREQUEST_METADATAENTRY']._serialized_end=543
  _globals['_REGISTERDEVICERESPONSE']._serialized_start=3599
  _globals['_REGISTERDEVICERESPONSE']._serialized_end=3695
  _globals['_UNREGISTERDEVICEREQUEST']._serialized_start=3697
  _globals['_UNREGISTERDEVICEREQUEST']._serialized_end=3758
  _globals['_UNREGISTERDEVICERESPONSE']._serialized_start=3760
  _globals['_UNREGISTERDEVICERESPONSE']._serialized_end=3820
  _globals['_LISTDEVICESREQUEST']._serialized_start=3823
  _globals['_LISTDEVICESREQUEST']._serialized_end=3958
  _globals['_LISTDEVICESRESPONSE']._serialized_start=3960
  _globals['_LISTDEVICESRESPONSE']._serialized_end=4074
  _globals['_GETDEVICEINFOREQUEST']._serialized_start=4076
  _globals['_GETDEVICEINFOREQUEST']._serialized_end=4134
  _globals['_GETDEVICEINFORESPONSE']._serialized_start=4136
  _globals['_GETDEVICEINFORESPONSE']._serialized_end=4197
  _globals['_UPDATEDEVICESTATUSREQUEST']._serialized_start=4200
  _globals['_UPDATEDEVICESTATUSREQUEST']._serialized_end=4421
  _globals['_UPDATEDEVICESTATUSREQUEST_METADATAENTRY']._serialized_start=496
  _globals['_UPDATEDEVICESTATUSREQUEST_METADATAENTRY']._serialized_end=543
  _globals['_UPDATEDEVICESTATUSRESPONSE']._serialized_start=4423
  _globals['_UPDATEDEVICESTATUSRESPONSE']._serialized_end=4506
  _globals['_GETTOPICINFOREQUEST']._serialized_start=4508
  _globals['_GETTOPICINFOREQUEST']._serialized_end=4561
  _globals['_GETTOPICINFORESPONSE']._serialized_start=4563
  _globals['_GETTOPICINFORESPONSE']._serialized_end=4626
  _globals['_LISTTOPICSREQUEST']._serialized_start=4628
  _globals['_LISTTOPICSREQUEST']._serialized_end=4738
  _globals['_LISTTOPICSRESPONSE']._serialized_start=4740
  _globals['_LISTTOPICSRESPONSE']._serialized_end=4818
  _globals['_VALIDATETOPICREQUEST']._serialized_start=4820
  _globals['_VALIDATETOPICREQUEST']._serialized_end=4882
  _globals['_VALIDATETOPICRESPONSE']._serialized_start=4884
  _globals['_VALIDATETOPICRESPONSE']._serialized_end=4959
  _globals['_SETRETAINEDMESSAGEREQUEST']._serialized_start=4961
  _globals['_SETRETAINEDMESSAGEREQUEST']._serialized_end=5070
  _globals['_SETRETAINEDMESSAGERESPONSE']._serialized_start=5072
  _globals['_SETRETAINEDMESSAGERESPONSE']._serialized_end=5134
  _globals['_GETRETAINEDMESSAGEREQUEST']._serialized_start=5136
  _globals['_GETRETAINEDMESSAGEREQUEST']._serialized_end=5195
  _globals['_GETRETAINEDMESSAGERESPONSE']._serialized_start=5197
  _globals['_GETRETAINEDMESSAGERESPONSE']._serialized_end=5280
  _globals['_DELETERETAINEDMESSAGEREQUEST']._serialized_start=5282
  _globals['_DELETERETAINEDMESSAGEREQUEST']._serialized_end=5344
  _globals['_DELETERETAINEDMESSAGERESPONSE']._serialized_start=5346
  _globals['_DELETERETAINEDMESSAGERESPONSE']._serialized_end=5411
  _globals['_GETSTATISTICSREQUEST']._serialized_start=5413
  _globals['_GETSTATISTICSREQUEST']._serialized_end=5477
  _globals['_GETSTATISTICSRESPONSE']._serialized_start=5480
  _globals['_GETSTATISTICSRESPONSE']._serialized_end=5846
  _globals['_GETSTATISTICSRESPONSE_DEVICETYPEDISTRIBUTIONENTRY']._serialized_start=5785
  _globals['_GETSTATISTICSRESPONSE_DEVICETYPEDISTRIBUTIONENTRY']._serialized_end=5846
  _globals['_GETDEVICEMETRICSREQUEST']._serialized_start=5849
  _globals['_GETDEVICEMETRICSREQUEST']._serialized_end=6004
  _globals['_GETDEVICEMETRICSRESPONSE']._serialized_start=6007
  _globals['_GETDEVICEMETRICSRESPONSE']._serialized_end=6242
  _globals['_TIMESERIESPOINT']._serialized_start=6244
  _globals['_TIMESERIESPOINT']._serialized_end=6323
  _globals['_MQTTHEALTHCHECKREQUEST']._serialized_start=6325
  _globals['_MQTTHEALTHCHECKREQUEST']._serialized_end=6369
  _globals['_MQTTHEALTHCHECKRESPONSE']._serialized_start=6372
  _globals['_MQTTHEALTHCHECKRESPONSE']._serialized_end=6530
  _globals['_SUBSCRIBEDEVICEMESSAGESREQUEST']._serialized_start=6533
  _globals['_SUBSCRIBEDEVICEMESSAGESREQUEST']._serialized_end=6703
  _globals['_DEVICEMESSAGE']._serialized_start=6706
  _globals['_DEVICEMESSAGE']._serialized_end=7009
  _globals['_DEVICEMESSAGE_METADATAENTRY']._serialized_start=496
  _globals['_DEVICEMESSAGE_METADATAENTRY']._serialized_end=543
  _globals['_WEBHOOKINFO']._serialized_start=7012
  _globals['_WEBHOOKINFO']._serialized_end=7431
  _globals['_WEBHOOKINFO_HEADERSENTRY']._serialized_start=7385
  _globals['_WEBHOOKINFO_HEADERSENTRY']._serialized_end=7431
  _globals['_REGISTERWEBHOOKREQUEST']._serialized_start=7434
  _globals['_REGISTERWEBHOOKREQUEST']._serialized_end=7737
  _globals['_REGISTERWEBHOOKREQUEST_HEADERSENTRY']._serialized_start=7385
  _globals['_REGISTERWEBHOOKREQUEST_HEADERSENTRY']._serialized_end=7431
  _globals['_REGISTERWEBHOOKRESPONSE']._serialized_start=7739
  _globals['_REGISTERWEBHOOKRESPONSE']._serialized_end=7858
  _globals['_UNREGISTERWEBHOOKREQUEST']._serialized_start=7860
  _globals['_UNREGISTERWEBHOOKREQUEST']._serialized_end=7923
  _globals['_UNREGISTERWEBHOOKRESPONSE']._serialized_start=7925
  _globals['_UNREGISTERWEBHOOKRESPONSE']._serialized_end=7986
  _globals['_LISTWEBHOOKSREQUEST']._serialized_start=7988
  _globals['_LISTWEBHOOKSREQUEST']._serialized_end=8077
  _globals['_LISTWEBHOOKSRESPONSE']._serialized_start=8079
  _globals['_LISTWEBHOOKSRESPONSE']._serialized_end=8163
  _globals['_MQTTSERVICE']._serialized_start=8598
  _globals['_MQTTSERVICE']._serialized_end=10962
# @@protoc_insertion_point(module_scope)



"""
Consul Service Registry Module
w
Provides service registration and health check functionality for microservices
"""

import asyncio
import json
import logging
import socket
from contextlib import asynccontextmanager
from typing import Any, Dict, List, Optional

import consul

logger = logging.getLogger(__name__)


class ConsulRegistry:
    """Handles service registration and discovery with Consul"""

    def __init__(
        self,
        service_name: Optional[str] = None,
        service_port: Optional[int] = None,
        consul_host: str = "localhost",
        consul_port: int = 8500,
        service_host: Optional[str] = None,
        tags: Optional[List[str]] = None,
        meta: Optional[Dict[str, str]] = None,
        health_check_type: str = "ttl",  # ttl or http
    ):
        """
        Initialize Consul registry

        Args:
            service_name: Name of the service to register (optional, required only for registration)
            service_port: Port the service is running on (optional, required only for registration)
            consul_host: Consul server host
            consul_port: Consul server port
            service_host: Service host (defaults to hostname)
            tags: Service tags for discovery
            meta: Service metadata for APISIX routing (e.g., {"api_path": "/api/v1/billing", "auth_required": "true"})
            health_check_type: Type of health check (ttl or http)

        Note:
            - For discovery-only usage, you can omit service_name and service_port
            - For registration, both service_name and service_port are required
            - meta can contain routing information for dynamic APISIX configuration
        """
        self.consul = consul.Consul(host=consul_host, port=consul_port)
        self.service_name = service_name
        self.service_port = service_port
        self.meta = meta or {}

        # Only set these if we're registering (have service_name and service_port)
        if service_name and service_port is not None:
            # Handle 0.0.0.0 which is invalid for Consul service registration
            # Priority: 1. Explicit service_host, 2. HOSTNAME env var, 3. socket.gethostname()
            # In Docker, HOSTNAME is set to the container name which is DNS resolvable
            import os

            if service_host and service_host != "0.0.0.0":
                self.service_host = service_host
            else:
                # Priority: SERVICE_HOST (K8s Service DNS) > HOSTNAME (Docker container name) > hostname
                # In K8s: SERVICE_HOST = "service.namespace.svc.cluster.local" (resolvable by Consul)
                # In Docker: SERVICE_HOST not set, falls back to HOSTNAME (container name, also resolvable)
                self.service_host = os.getenv("SERVICE_HOST") or os.getenv(
                    "HOSTNAME", socket.gethostname()
                )
            self.service_id = f"{service_name}-{self.service_host}-{service_port}"
        else:
            import os

            self.service_host = service_host or os.getenv(
                "HOSTNAME", socket.gethostname()
            )
            self.service_id = None  # No service ID for discovery-only mode

        self.tags = tags or []
        self.check_interval = "15s"
        self.deregister_after = "90s"  # 增加到90秒，给服务更多时间
        self._health_check_task = None
        self.health_check_type = health_check_type
        self.ttl_interval = 30  # 标准30秒TTL间隔

    def cleanup_stale_registrations(self) -> int:
        """
        Clean up stale registrations for this service before registering.
        Removes any existing registrations with different hostnames (e.g., old container IDs).

        Returns:
            Number of stale registrations removed
        """
        try:
            removed_count = 0
            services = self.consul.agent.services()

            # Find all registrations for this service name on the same port
            for service_id, service_info in services.items():
                if (
                    service_info["Service"] == self.service_name
                    and service_info["Port"] == self.service_port
                    and service_id != self.service_id
                ):  # Different service_id = stale
                    logger.info(
                        f"🧹 Removing stale registration: {service_id} (address: {service_info['Address']})"
                    )
                    self.consul.agent.service.deregister(service_id)
                    removed_count += 1

            if removed_count > 0:
                logger.info(
                    f"✨ Cleaned up {removed_count} stale registration(s) for {self.service_name}"
                )

            return removed_count

        except Exception as e:
            logger.warning(f"⚠️  Failed to cleanup stale registrations: {e}")
            return 0

    def register(self, cleanup_stale: bool = True) -> bool:
        """
        Register service with Consul

        Args:
            cleanup_stale: If True, remove stale registrations before registering (default: True)
        """
        try:
            # Clean up stale registrations first
            if cleanup_stale:
                self.cleanup_stale_registrations()

            # Choose health check type
            if self.health_check_type == "ttl":
                check = consul.Check.ttl(f"{self.ttl_interval}s")
            else:
                check = consul.Check.http(
                    f"http://{self.service_host}:{self.service_port}/health",
                    interval=self.check_interval,
                    timeout="5s",
                    deregister=self.deregister_after,
                )

            # Register service with selected health check
            self.consul.agent.service.register(
                name=self.service_name,
                service_id=self.service_id,
                address=self.service_host,
                port=self.service_port,
                tags=self.tags,
                meta=self.meta,
                check=check,
            )

            # If TTL, immediately pass the health check
            if self.health_check_type == "ttl":
                self.consul.agent.check.ttl_pass(f"service:{self.service_id}")

            logger.info(
                f"✅ Service registered with Consul: {self.service_name} "
                f"({self.service_id}) at {self.service_host}:{self.service_port} "
                f"with {self.health_check_type} health check, tags: {self.tags}"
            )
            self._log_service_metrics("register", True)
            return True

        except Exception as e:
            logger.error(f"❌ Failed to register service with Consul: {e}")
            self._log_service_metrics("register", False)
            return False

    def deregister(self) -> bool:
        """Deregister service from Consul"""
        try:
            self.consul.agent.service.deregister(self.service_id)
            logger.info(f"✅ Service deregistered from Consul: {self.service_id}")
            self._log_service_metrics("deregister", True)
            return True

        except Exception as e:
            logger.error(f"❌ Failed to deregister service from Consul: {e}")
            self._log_service_metrics("deregister", False)
            return False

    async def maintain_registration(self):
        """Maintain service registration (re-register if needed)"""
        retry_count = 0
        max_retries = 3

        while True:
            try:
                # Check if service is still registered
                services = self.consul.agent.services()
                if self.service_id not in services:
                    logger.warning(
                        f"Service {self.service_id} not found in Consul, re-registering..."
                    )
                    if self.register():
                        retry_count = 0  # 重置重试计数
                        logger.info(f"Successfully re-registered {self.service_id}")
                    else:
                        retry_count += 1
                        logger.error(
                            f"Failed to re-register {self.service_id}, retry {retry_count}/{max_retries}"
                        )

                # If using TTL checks, update the health status
                if self.health_check_type == "ttl":
                    try:
                        self.consul.agent.check.ttl_pass(
                            f"service:{self.service_id}",
                            f"Service is healthy - {self.service_name}@{self.service_host}:{self.service_port}",
                        )
                        logger.debug(f"TTL health check passed for {self.service_id}")
                        retry_count = 0  # TTL成功则重置重试计数
                    except Exception as e:
                        retry_count += 1
                        logger.warning(
                            f"Failed to update TTL health check: {e}, retry {retry_count}/{max_retries}"
                        )

                        # 如果TTL连续失败，尝试重新注册
                        if retry_count >= max_retries:
                            logger.error(
                                f"TTL failed {max_retries} times, attempting re-registration"
                            )
                            self.register()
                            retry_count = 0

                # 动态调整睡眠时间，错误时更频繁检查
                if retry_count > 0:
                    sleep_time = 5  # 有错误时5秒检查一次
                else:
                    sleep_time = (
                        self.ttl_interval / 2 if self.health_check_type == "ttl" else 30
                    )

                await asyncio.sleep(sleep_time)

            except asyncio.CancelledError:
                break
            except Exception as e:
                retry_count += 1
                logger.error(
                    f"Error maintaining registration: {e}, retry {retry_count}/{max_retries}"
                )
                # 指数退避
                sleep_time = min(10 * (2 ** (retry_count - 1)), 60)
                await asyncio.sleep(sleep_time)

    def start_maintenance(self):
        """Start the background maintenance task"""
        if not self._health_check_task:
            loop = asyncio.get_event_loop()
            self._health_check_task = loop.create_task(self.maintain_registration())

    def stop_maintenance(self):
        """Stop the background maintenance task"""
        if self._health_check_task:
            self._health_check_task.cancel()
            self._health_check_task = None

    # Configuration Management Methods
    def get_config(self, key: str, default: Any = None) -> Any:
        """Get configuration value from Consul KV store"""
        try:
            full_key = f"{self.service_name}/{key}"
            index, data = self.consul.kv.get(full_key)
            if data and data.get("Value"):
                value = data["Value"].decode("utf-8")
                # Try to parse as JSON
                try:
                    return json.loads(value)
                except json.JSONDecodeError:
                    return value
            return default
        except Exception as e:
            logger.error(f"Failed to get config {key}: {e}")
            return default

    def set_config(self, key: str, value: Any) -> bool:
        """Set configuration value in Consul KV store"""
        try:
            full_key = f"{self.service_name}/{key}"
            # Convert to JSON if not string
            if not isinstance(value, str):
                value = json.dumps(value)
            return self.consul.kv.put(full_key, value)
        except Exception as e:
            logger.error(f"Failed to set config {key}: {e}")
            return False

    def get_all_config(self) -> Dict[str, Any]:
        """Get all configuration for this service"""
        try:
            prefix = f"{self.service_name}/"
            index, data = self.consul.kv.get(prefix, recurse=True)
            if not data:
                return {}

            config = {}
            for item in data:
                if item["Value"]:
                    key = item["Key"].replace(prefix, "")
                    value = item["Value"].decode("utf-8")
                    try:
                        config[key] = json.loads(value)
                    except json.JSONDecodeError:
                        config[key] = value
            return config
        except Exception as e:
            logger.error(f"Failed to get all config: {e}")
            return {}

    def watch_config(self, key: str, callback):
        """Watch for configuration changes (blocking call)"""
        full_key = f"{self.service_name}/{key}"
        index = None
        while True:
            try:
                index, data = self.consul.kv.get(full_key, index=index, wait="30s")
                if data:
                    value = data["Value"].decode("utf-8")
                    try:
                        value = json.loads(value)
                    except json.JSONDecodeError:
                        pass
                    callback(key, value)
            except Exception as e:
                logger.error(f"Error watching config {key}: {e}")
                break

    # Service Discovery Methods
    def discover_service(self, service_name: str) -> List[Dict[str, Any]]:
        """Discover healthy instances of a service"""
        try:
            # Get health checks for the service
            index, services = self.consul.health.service(service_name, passing=True)

            instances = []
            for service in services:
                instance = {
                    "id": service["Service"]["ID"],
                    "address": service["Service"]["Address"],
                    "port": service["Service"]["Port"],
                    "tags": service["Service"].get("Tags", []),
                    "meta": service["Service"].get("Meta", {}),
                }
                instances.append(instance)

            return instances
        except Exception as e:
            logger.error(f"Failed to discover service {service_name}: {e}")
            return []

    def get_service_endpoint(
        self, service_name: str, strategy: str = "health_weighted"
    ) -> Optional[str]:
        """Get a single service endpoint using advanced load balancing strategy"""
        instances = self.discover_service(service_name)
        if not instances:
            return None

        # 只有一个实例时直接返回
        if len(instances) == 1:
            instance = instances[0]
            return f"http://{instance['address']}:{instance['port']}"

        # 高级负载均衡策略
        if strategy == "health_weighted":
            # 基于健康状态和权重选择最佳实例
            instance = self._select_best_instance(instances)
        elif strategy == "random":
            import random

            instance = random.choice(instances)
        elif strategy == "round_robin":
            # 实现真正的轮询（使用实例缓存）
            instance = self._get_round_robin_instance(service_name, instances)
        elif strategy == "least_connections":
            # 选择连接数最少的实例（模拟实现）
            instance = min(instances, key=lambda x: hash(x["id"]) % 100)
        else:
            # 默认随机选择
            import random

            instance = random.choice(instances)

        return f"http://{instance['address']}:{instance['port']}"

    def _select_best_instance(self, instances: List[Dict[str, Any]]) -> Dict[str, Any]:
        """选择最佳实例（基于健康状态和负载）"""
        # 简单实现：优先选择标签包含'preferred'的实例
        preferred_instances = [
            inst for inst in instances if "preferred" in inst.get("tags", [])
        ]
        if preferred_instances:
            import random

            return random.choice(preferred_instances)

        # 没有首选实例时随机选择
        import random

        return random.choice(instances)

    def _get_round_robin_instance(
        self, service_name: str, instances: List[Dict[str, Any]]
    ) -> Dict[str, Any]:
        """实现真正的轮询负载均衡"""
        if not hasattr(self, "_round_robin_counters"):
            self._round_robin_counters = {}

        if service_name not in self._round_robin_counters:
            self._round_robin_counters[service_name] = 0

        # 获取当前计数器并递增
        counter = self._round_robin_counters[service_name]
        self._round_robin_counters[service_name] = (counter + 1) % len(instances)

        return instances[counter]

    def _log_service_metrics(
        self, operation: str, success: bool, service_name: str = None
    ):
        """记录服务操作指标"""
        service = service_name or self.service_name
        status = "SUCCESS" if success else "FAILED"

        # 使用项目统一的logger记录指标
        logger.info(
            f"🔍 CONSUL_METRICS | operation={operation} | service={service} | "
            f"status={status} | service_id={self.service_id}"
        )

    def get_service_address(
        self,
        service_name: str,
        fallback_url: Optional[str] = None,
        max_retries: int = 3,
    ) -> str:
        """
        Get service address from Consul discovery with automatic fallback and retry

        Args:
            service_name: Name of the service to discover
            fallback_url: Fallback URL if service not found in Consul (e.g., "http://localhost:8201")
            max_retries: Maximum number of discovery attempts

        Returns:
            Service URL (from Consul or fallback)

        Example:
            consul = ConsulRegistry("my_service", 8080)
            url = consul.get_service_address("account_service", "http://localhost:8201")
            # Returns: "http://10.0.1.5:8201" (from Consul) or "http://localhost:8201" (fallback)
        """
        last_error = None

        for attempt in range(max_retries):
            try:
                endpoint = self.get_service_endpoint(service_name)
                if endpoint:
                    logger.debug(
                        f"Discovered {service_name} at {endpoint} (attempt {attempt + 1})"
                    )
                    return endpoint

                # 如果没找到服务但没有异常，记录并继续
                last_error = f"Service {service_name} not found in Consul registry"

            except Exception as e:
                last_error = e
                logger.warning(
                    f"Consul discovery attempt {attempt + 1} failed for {service_name}: {e}"
                )

                # 短暂等待后重试（除了最后一次）
                if attempt < max_retries - 1:
                    import time

                    time.sleep(0.5 * (attempt + 1))  # 递增延迟

        # 所有重试都失败，使用fallback
        if fallback_url:
            logger.warning(
                f"All {max_retries} discovery attempts failed for {service_name}: {last_error}, using fallback: {fallback_url}"
            )
            return fallback_url

        raise ValueError(
            f"Service {service_name} not found after {max_retries} attempts and no fallback provided. Last error: {last_error}"
        )

    def watch_service(self, service_name: str, callback, wait_time: str = "30s"):
        """Watch for changes in service instances"""
        index = None
        while True:
            try:
                index, services = self.consul.health.service(
                    service_name, passing=True, index=index, wait=wait_time
                )
                # Convert to simplified format
                instances = []
                for service in services:
                    instances.append(
                        {
                            "id": service["Service"]["ID"],
                            "address": service["Service"]["Address"],
                            "port": service["Service"]["Port"],
                        }
                    )
                callback(service_name, instances)
            except Exception as e:
                logger.error(f"Error watching service {service_name}: {e}")
                break

    # ============================================
    # Convenience Methods for Service Discovery
    # ============================================

    def get_auth_service_url(self) -> str:
        """Get auth service URL"""
        endpoint = self.get_service_endpoint("auth_service")
        if not endpoint:
            raise ValueError("Service auth_service not found in Consul")
        return endpoint

    def get_payment_service_url(self) -> str:
        """Get payment service URL"""
        endpoint = self.get_service_endpoint("payment_service")
        if not endpoint:
            raise ValueError("Service payment_service not found in Consul")
        return endpoint

    def get_storage_service_url(self) -> str:
        """Get storage service URL"""
        endpoint = self.get_service_endpoint("storage_service")
        if not endpoint:
            raise ValueError("Service storage_service not found in Consul")
        return endpoint

    def get_notification_service_url(self) -> str:
        """Get notification service URL"""
        endpoint = self.get_service_endpoint("notification_service")
        if not endpoint:
            raise ValueError("Service notification_service not found in Consul")
        return endpoint

    def get_account_service_url(self) -> str:
        """Get account service URL"""
        endpoint = self.get_service_endpoint("account_service")
        if not endpoint:
            raise ValueError("Service account_service not found in Consul")
        return endpoint

    def get_session_service_url(self) -> str:
        """Get session service URL"""
        endpoint = self.get_service_endpoint("session_service")
        if not endpoint:
            raise ValueError("Service session_service not found in Consul")
        return endpoint

    def get_order_service_url(self) -> str:
        """Get order service URL"""
        endpoint = self.get_service_endpoint("order_service")
        if not endpoint:
            raise ValueError("Service order_service not found in Consul")
        return endpoint

    def get_task_service_url(self) -> str:
        """Get task service URL"""
        endpoint = self.get_service_endpoint("task_service")
        if not endpoint:
            raise ValueError("Service task_service not found in Consul")
        return endpoint

    def get_device_service_url(self) -> str:
        """Get device service URL"""
        endpoint = self.get_service_endpoint("device_service")
        if not endpoint:
            raise ValueError("Service device_service not found in Consul")
        return endpoint

    def get_organization_service_url(self) -> str:
        """Get organization service URL"""
        endpoint = self.get_service_endpoint("organization_service")
        if not endpoint:
            raise ValueError("Service organization_service not found in Consul")
        return endpoint

    # Infrastructure Services Discovery
    def get_nats_url(self) -> str:
        """Get NATS message queue URL"""
        endpoint = self.get_service_endpoint("nats-grpc-service")
        if not endpoint:
            raise ValueError("Service nats-grpc-service not found in Consul")
        return endpoint

    def get_redis_url(self) -> str:
        """Get Redis cache URL"""
        endpoint = self.get_service_endpoint("redis-grpc-service")
        if not endpoint:
            raise ValueError("Service redis-grpc-service not found in Consul")
        return endpoint

    def get_loki_url(self) -> str:
        """Get Loki logging service URL"""
        endpoint = self.get_service_endpoint("loki-grpc-service")
        if not endpoint:
            raise ValueError("Service loki-grpc-service not found in Consul")
        return endpoint

    def get_minio_endpoint(self) -> str:
        """Get MinIO object storage endpoint"""
        endpoint = self.get_service_endpoint("minio-grpc-service")
        if not endpoint:
            raise ValueError("Service minio-grpc-service not found in Consul")
        return endpoint

    def get_duckdb_url(self) -> str:
        """Get DuckDB service URL"""
        endpoint = self.get_service_endpoint("duckdb-grpc-service")
        if not endpoint:
            raise ValueError("Service duckdb-grpc-service not found in Consul")
        return endpoint


@asynccontextmanager
async def consul_lifespan(
    app,
    service_name: str,
    service_port: int,
    consul_host: str = "localhost",
    consul_port: int = 8500,
    tags: Optional[List[str]] = None,
    meta: Optional[Dict[str, str]] = None,
    health_check_type: str = "ttl",
):
    """
    FastAPI lifespan context manager for Consul registration

    Usage:
        app = FastAPI(lifespan=lambda app: consul_lifespan(
            app,
            "my-service",
            8080,
            meta={"api_path": "/api/v1/myservice", "auth_required": "true"}
        ))
    """
    # Startup
    # Use SERVICE_HOST env var if available, otherwise use hostname
    import os

    service_host = os.getenv("SERVICE_HOST", socket.gethostname())

    registry = ConsulRegistry(
        service_name=service_name,
        service_port=service_port,
        consul_host=consul_host,
        consul_port=consul_port,
        service_host=service_host,  # Use SERVICE_HOST from env or hostname
        tags=tags,
        meta=meta,
        health_check_type=health_check_type,
    )

    # Register with Consul
    if registry.register():
        # Start maintenance task
        registry.start_maintenance()
        # Store in app state for access in routes
        app.state.consul_registry = registry
    else:
        logger.warning(
            "Failed to register with Consul, continuing without service discovery"
        )

    yield

    # Shutdown
    if hasattr(app.state, "consul_registry"):
        registry.stop_maintenance()
        registry.deregister()



#!/usr/bin/env python3
"""
Redis gRPC Client
Redis cache client
"""

from typing import List, Dict, Optional, TYPE_CHECKING
from .base_client import BaseGRPCClient
from .proto import redis_service_pb2, redis_service_pb2_grpc

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry


class RedisClient(BaseGRPCClient):
    """Redis gRPC Client"""

    def __init__(self, host: Optional[str] = None, port: Optional[int] = None, user_id: Optional[str] = None,
                 organization_id: Optional[str] = None, lazy_connect: bool = True,
                 enable_compression: bool = True, enable_retry: bool = True,
                 consul_registry: Optional['ConsulRegistry'] = None, service_name_override: Optional[str] = None):
        """
        Initialize Redis client

        Args:
            host: Service host (optional, will use Consul discovery if not provided)
            port: Service port (optional, will use Consul discovery if not provided)
            user_id: User ID
            organization_id: Organization ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: True)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional, defaults to 'redis')
        """
        # Let BaseGRPCClient handle Consul discovery and fallback defaults
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )
        self.organization_id = organization_id or 'default-org'

    def _create_stub(self):
        """Create Redis service stub"""
        return redis_service_pb2_grpc.RedisServiceStub(self.channel)

    def service_name(self) -> str:
        return "Redis"

    def default_port(self) -> int:
        return 50055

    def health_check(self) -> Optional[Dict]:
        """Health check"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.RedisHealthCheckRequest(
                deep_check=False
            )
            response = self.stub.HealthCheck(request)

            return {
                'healthy': response.healthy,
                'redis_status': response.redis_status,
                'connected_clients': response.connected_clients,
                'used_memory_bytes': response.used_memory_bytes
            }

        except Exception as e:
            return self.handle_error(e, "Health check")

    def set(self, key: str, value: str, ttl_seconds: int = 0) -> Optional[bool]:
        """Set key-value"""
        try:
            self._ensure_connected()

            # Use SetWithExpirationRequest if TTL is provided
            if ttl_seconds > 0:
                from google.protobuf.duration_pb2 import Duration
                duration = Duration()
                duration.seconds = ttl_seconds

                request = redis_service_pb2.SetWithExpirationRequest(
                    user_id=self.user_id,
                    organization_id=self.organization_id,
                    key=key,
                    value=value,
                    expiration=duration
                )
                response = self.stub.SetWithExpiration(request)
            else:
                request = redis_service_pb2.SetRequest(
                    user_id=self.user_id,
                    organization_id=self.organization_id,
                    key=key,
                    value=value
                )
                response = self.stub.Set(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Set key-value")
            return False

    def get(self, key: str) -> Optional[str]:
        """Get value by key"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.GetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.Get(request)

            if response.found:
                return response.value
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Get key-value")

    def delete(self, key: str) -> Optional[bool]:
        """Delete key"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.DeleteRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.Delete(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Delete key")
            return False

    def exists(self, key: str) -> bool:
        """Check if key exists"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ExistsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.Exists(request)

            return response.exists

        except Exception as e:
            self.handle_error(e, "Check key exists")
            return False

    def set_with_ttl(self, key: str, value: str, ttl_seconds: int) -> Optional[bool]:
        """Set key-value with TTL"""
        return self.set(key, value, ttl_seconds)

    def append(self, key: str, value: str) -> Optional[int]:
        """Append value to key"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.AppendRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                value=value
            )

            response = self.stub.Append(request)

            return response.length

        except Exception as e:
            return self.handle_error(e, "Append to key")

    def mset(self, key_values: Dict[str, str], ttl_seconds: int = 0) -> Optional[bool]:
        """Batch set key-values using ExecuteBatch for optimal performance

        Args:
            key_values: Dictionary of key-value pairs to set
            ttl_seconds: Optional TTL for all keys (0 = no expiration)

        Returns:
            True if all keys were set successfully, False otherwise
        """
        try:
            self._ensure_connected()

            # Build batch commands for all key-value pairs
            batch_commands = []
            for key, value in key_values.items():
                cmd = redis_service_pb2.BatchCommand(
                    operation='SET',
                    key=key,
                    value=value
                )
                # Add expiration if specified
                if ttl_seconds > 0:
                    from google.protobuf.duration_pb2 import Duration
                    expiration = Duration()
                    expiration.seconds = ttl_seconds
                    cmd.expiration.CopyFrom(expiration)
                batch_commands.append(cmd)

            # Execute all SET operations in a single RPC call
            request = redis_service_pb2.RedisBatchRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                commands=batch_commands
            )

            response = self.stub.ExecuteBatch(request)

            if response.success and response.executed_count == len(key_values):
                return True
            else:
                if response.errors:
                    for error in response.errors:
                        self.handle_error(Exception(error), "Batch set")
                return False

        except Exception as e:
            self.handle_error(e, "Batch set key-values")
            return False

    def mget(self, keys: List[str]) -> Dict[str, str]:
        """Batch get key-values"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.GetMultipleRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                keys=keys
            )

            response = self.stub.GetMultiple(request)

            # Convert KeyValue list to dict
            values = {kv.key: kv.value for kv in response.values}
            return values

        except Exception as e:
            return self.handle_error(e, "Batch get key-values") or {}

    def incr(self, key: str, delta: int = 1) -> Optional[int]:
        """Increment"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.IncrementRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                delta=delta
            )

            response = self.stub.Increment(request)

            return response.value

        except Exception as e:
            return self.handle_error(e, "Increment")

    def decr(self, key: str, delta: int = 1) -> Optional[int]:
        """Decrement"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.DecrementRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                delta=delta
            )

            response = self.stub.Decrement(request)

            return response.value

        except Exception as e:
            return self.handle_error(e, "Decrement")

    def expire(self, key: str, seconds: int) -> Optional[bool]:
        """Set expiration time"""
        try:
            self._ensure_connected()
            
            from google.protobuf.duration_pb2 import Duration
            duration = Duration()
            duration.seconds = seconds
            
            request = redis_service_pb2.ExpireRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                expiration=duration
            )

            response = self.stub.Expire(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Set expiration")
            return False

    def ttl(self, key: str) -> Optional[int]:
        """Get time to live (seconds)"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.GetTTLRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.GetTTL(request)

            return response.ttl_seconds

        except Exception as e:
            return self.handle_error(e, "Get TTL")

    def delete_multiple(self, keys: List[str]) -> Optional[int]:
        """Delete multiple keys"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.DeleteMultipleRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                keys=keys
            )

            response = self.stub.DeleteMultiple(request)

            if response.success:
                return response.deleted_count
            else:
                return 0

        except Exception as e:
            self.handle_error(e, "Delete multiple keys")
            return 0

    def rename(self, old_key: str, new_key: str) -> Optional[bool]:
        """Rename key"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.RenameRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                old_key=old_key,
                new_key=new_key
            )

            response = self.stub.Rename(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Rename key")
            return False

    def list_keys(self, pattern: str = "*", limit: int = 100) -> List[str]:
        """List keys matching pattern"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ListKeysRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                pattern=pattern,
                limit=limit
            )

            response = self.stub.ListKeys(request)

            return list(response.keys)

        except Exception as e:
            return self.handle_error(e, "List keys") or []

    def lpush(self, key: str, values: List[str]) -> Optional[int]:
        """Left push to list"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.LPushRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                values=values
            )

            response = self.stub.LPush(request)

            return response.length

        except Exception as e:
            return self.handle_error(e, "Left push to list")

    def rpush(self, key: str, values: List[str]) -> Optional[int]:
        """Right push to list"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.RPushRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                values=values
            )

            response = self.stub.RPush(request)

            return response.length

        except Exception as e:
            return self.handle_error(e, "Right push to list")

    def lrange(self, key: str, start: int = 0, stop: int = -1) -> List[str]:
        """Get list range"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.LRangeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                start=start,
                stop=stop
            )

            response = self.stub.LRange(request)

            return list(response.values)

        except Exception as e:
            return self.handle_error(e, "Get list range") or []

    def lpop(self, key: str) -> Optional[str]:
        """Pop element from left of list"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.LPopRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.LPop(request)

            if response.found:
                return response.value
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Left pop from list")

    def rpop(self, key: str) -> Optional[str]:
        """Pop element from right of list"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.RPopRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.RPop(request)

            if response.found:
                return response.value
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Right pop from list")

    def llen(self, key: str) -> Optional[int]:
        """Get list length"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.LLenRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.LLen(request)

            return response.length

        except Exception as e:
            return self.handle_error(e, "Get list length")

    def lindex(self, key: str, index: int) -> Optional[str]:
        """Get list element at index"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.LIndexRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                index=index
            )

            response = self.stub.LIndex(request)

            if response.found:
                return response.value
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Get list element")

    def ltrim(self, key: str, start: int, stop: int) -> Optional[bool]:
        """Trim list to specified range"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.LTrimRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                start=start,
                stop=stop
            )

            response = self.stub.LTrim(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Trim list")
            return False

    def hset(self, key: str, field: str, value: str) -> Optional[bool]:
        """Set hash field"""
        try:
            self._ensure_connected()

            # Create HashField message
            hash_field = redis_service_pb2.HashField(field=field, value=value)

            request = redis_service_pb2.HSetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                fields=[hash_field]
            )

            response = self.stub.HSet(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Set hash field")
            return False

    def hget(self, key: str, field: str) -> Optional[str]:
        """Get hash field"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.HGetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                field=field
            )

            response = self.stub.HGet(request)

            if response.found:
                return response.value
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Get hash field")

    def hgetall(self, key: str) -> Dict[str, str]:
        """Get all hash fields"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.HGetAllRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.HGetAll(request)

            # Convert repeated HashField to dict
            fields = {f.field: f.value for f in response.fields}
            return fields

        except Exception as e:
            return self.handle_error(e, "Get all hash fields") or {}

    def hdelete(self, key: str, fields: List[str]) -> Optional[int]:
        """Delete hash fields"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.HDeleteRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                fields=fields
            )

            response = self.stub.HDelete(request)

            if response.success:
                return response.deleted_count
            else:
                return 0

        except Exception as e:
            self.handle_error(e, "Delete hash fields")
            return 0

    def hexists(self, key: str, field: str) -> bool:
        """Check if hash field exists"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.HExistsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                field=field
            )

            response = self.stub.HExists(request)

            return response.exists

        except Exception as e:
            self.handle_error(e, "Check hash field exists")
            return False

    def hkeys(self, key: str) -> List[str]:
        """Get all hash field names"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.HKeysRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.HKeys(request)

            return list(response.fields)

        except Exception as e:
            return self.handle_error(e, "Get hash keys") or []

    def hvalues(self, key: str) -> List[str]:
        """Get all hash values"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.HValuesRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.HValues(request)

            return list(response.values)

        except Exception as e:
            return self.handle_error(e, "Get hash values") or []

    def hincrement(self, key: str, field: str, delta: int = 1) -> Optional[int]:
        """Increment hash field value"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.HIncrementRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                field=field,
                delta=delta
            )

            response = self.stub.HIncrement(request)

            return response.value

        except Exception as e:
            return self.handle_error(e, "Increment hash field")

    # ============================================
    # Set Operations
    # ============================================

    def sadd(self, key: str, members: List[str]) -> Optional[int]:
        """Add members to set"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SAddRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = self.stub.SAdd(request)

            return response.added_count

        except Exception as e:
            return self.handle_error(e, "Add to set")

    def sremove(self, key: str, members: List[str]) -> Optional[int]:
        """Remove members from set"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SRemoveRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = self.stub.SRemove(request)

            return response.removed_count

        except Exception as e:
            return self.handle_error(e, "Remove from set")

    def smembers(self, key: str) -> List[str]:
        """Get all set members"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SMembersRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.SMembers(request)

            return list(response.members)

        except Exception as e:
            return self.handle_error(e, "Get set members") or []

    def sismember(self, key: str, member: str) -> bool:
        """Check if member is in set"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SIsMemberRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                member=member
            )

            response = self.stub.SIsMember(request)

            return response.is_member

        except Exception as e:
            self.handle_error(e, "Check set membership")
            return False

    def scard(self, key: str) -> Optional[int]:
        """Get set cardinality (size)"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SCardRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.SCard(request)

            return response.count

        except Exception as e:
            return self.handle_error(e, "Get set cardinality")

    def sunion(self, keys: List[str]) -> List[str]:
        """Get union of sets"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SUnionRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                keys=keys
            )

            response = self.stub.SUnion(request)

            return list(response.members)

        except Exception as e:
            return self.handle_error(e, "Get set union") or []

    def sinter(self, keys: List[str]) -> List[str]:
        """Get intersection of sets"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SInterRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                keys=keys
            )

            response = self.stub.SInter(request)

            return list(response.members)

        except Exception as e:
            return self.handle_error(e, "Get set intersection") or []

    def sdiff(self, keys: List[str]) -> List[str]:
        """Get difference of sets"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.SDiffRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                keys=keys
            )

            response = self.stub.SDiff(request)

            return list(response.members)

        except Exception as e:
            return self.handle_error(e, "Get set difference") or []

    # ============================================
    # Sorted Set Operations
    # ============================================

    def zadd(self, key: str, score_members: Dict[str, float]) -> Optional[int]:
        """
        Add members to sorted set with scores

        Args:
            key: Sorted set key
            score_members: Dict of {member: score}

        Returns:
            Number of members added
        """
        try:
            self._ensure_connected()

            # Create ZSetMember messages
            members = [
                redis_service_pb2.ZSetMember(member=member, score=score)
                for member, score in score_members.items()
            ]

            request = redis_service_pb2.ZAddRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = self.stub.ZAdd(request)

            return response.added_count

        except Exception as e:
            self.handle_error(e, "Add to sorted set")
            return None

    def zrange(self, key: str, start: int = 0, stop: int = -1, with_scores: bool = False) -> List:
        """
        Get sorted set range

        Args:
            key: Sorted set key
            start: Start index
            stop: Stop index (-1 for end)
            with_scores: Return scores with members

        Returns:
            List of members or list of (member, score) tuples if with_scores=True
        """
        try:
            self._ensure_connected()
            request = redis_service_pb2.ZRangeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                start=start,
                stop=stop,
                with_scores=with_scores
            )

            response = self.stub.ZRange(request)

            if with_scores:
                result = [(m.member, m.score) for m in response.members]
            else:
                result = [m.member for m in response.members]

            return result

        except Exception as e:
            return self.handle_error(e, "Get sorted set range") or []

    def zrem(self, key: str, members: List[str]) -> Optional[int]:
        """
        Remove members from sorted set

        Args:
            key: Sorted set key
            members: List of members to remove

        Returns:
            Number of members removed
        """
        try:
            self._ensure_connected()
            request = redis_service_pb2.ZRemoveRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = self.stub.ZRemove(request)

            return response.removed_count

        except Exception as e:
            self.handle_error(e, "Remove from sorted set")
            return None

    def zremrangebyrank(self, key: str, start: int, stop: int) -> Optional[int]:
        """
        Remove sorted set members by rank range

        Args:
            key: Sorted set key
            start: Start rank
            stop: Stop rank

        Returns:
            Number of members removed
        """
        try:
            # Get members in range first
            members_to_remove = self.zrange(key, start, stop, with_scores=False)

            if not members_to_remove:
                return 0

            # Remove them
            return self.zrem(key, members_to_remove)

        except Exception as e:
            self.handle_error(e, "Remove sorted set by rank")
            return None

    def zrange_by_score(self, key: str, min_score: float, max_score: float, 
                        offset: int = 0, count: int = -1) -> List[tuple]:
        """
        Get sorted set members by score range

        Args:
            key: Sorted set key
            min_score: Minimum score
            max_score: Maximum score
            offset: Offset for pagination
            count: Number of results (-1 for all)

        Returns:
            List of (member, score) tuples
        """
        try:
            self._ensure_connected()
            request = redis_service_pb2.ZRangeByScoreRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                min_score=min_score,
                max_score=max_score,
                offset=offset,
                count=count
            )

            response = self.stub.ZRangeByScore(request)

            result = [(m.member, m.score) for m in response.members]
            return result

        except Exception as e:
            return self.handle_error(e, "Get sorted set by score") or []

    def zrank(self, key: str, member: str) -> Optional[int]:
        """Get member rank in sorted set"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ZRankRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                member=member
            )

            response = self.stub.ZRank(request)

            if response.found:
                return response.rank
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Get sorted set rank")

    def zscore(self, key: str, member: str) -> Optional[float]:
        """Get member score in sorted set"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ZScoreRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                member=member
            )

            response = self.stub.ZScore(request)

            if response.found:
                return response.score
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Get sorted set score")

    def zcard(self, key: str) -> Optional[int]:
        """Get sorted set cardinality (size)"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ZCardRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.ZCard(request)

            return response.count

        except Exception as e:
            return self.handle_error(e, "Get sorted set cardinality")

    def zincrement(self, key: str, member: str, delta: float = 1.0) -> Optional[float]:
        """Increment member score in sorted set"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ZIncrementRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                member=member,
                delta=delta
            )

            response = self.stub.ZIncrement(request)

            return response.score

        except Exception as e:
            return self.handle_error(e, "Increment sorted set score")


    # ============================================
    # Distributed Lock Operations
    # ============================================

    def acquire_lock(self, lock_key: str, ttl_seconds: int = 10, wait_timeout_seconds: int = 5) -> Optional[str]:
        """
        Acquire distributed lock

        Args:
            lock_key: Lock key
            ttl_seconds: Lock TTL in seconds
            wait_timeout_seconds: Wait timeout in seconds

        Returns:
            lock_id if successful, None otherwise
        """
        try:
            self._ensure_connected()
            
            from google.protobuf.duration_pb2 import Duration
            ttl = Duration()
            ttl.seconds = ttl_seconds
            wait_timeout = Duration()
            wait_timeout.seconds = wait_timeout_seconds
            
            request = redis_service_pb2.AcquireLockRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                lock_key=lock_key,
                ttl=ttl,
                wait_timeout=wait_timeout
            )

            response = self.stub.AcquireLock(request)

            if response.acquired:
                return response.lock_id
            else:
                return None

        except Exception as e:
            self.handle_error(e, "Acquire lock")
            return None

    def release_lock(self, lock_key: str, lock_id: str) -> bool:
        """Release distributed lock"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ReleaseLockRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                lock_key=lock_key,
                lock_id=lock_id
            )

            response = self.stub.ReleaseLock(request)

            if response.released:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Release lock")
            return False

    def renew_lock(self, lock_key: str, lock_id: str, ttl_seconds: int = 10) -> bool:
        """Renew distributed lock"""
        try:
            self._ensure_connected()
            
            from google.protobuf.duration_pb2 import Duration
            ttl = Duration()
            ttl.seconds = ttl_seconds
            
            request = redis_service_pb2.RenewLockRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                lock_key=lock_key,
                lock_id=lock_id,
                ttl=ttl
            )

            response = self.stub.RenewLock(request)

            if response.renewed:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Renew lock")
            return False

    # ============================================
    # Pub/Sub Operations
    # ============================================

    def publish(self, channel: str, message: str) -> Optional[int]:
        """Publish message to channel"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.PublishRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                channel=channel,
                message=message
            )

            response = self.stub.Publish(request)

            return response.subscriber_count

        except Exception as e:
            return self.handle_error(e, "Publish message")

    def subscribe(self, channels: List[str], callback=None):
        """
        Subscribe to channels (streaming)
        
        Args:
            channels: List of channel names
            callback: Function to call for each message (takes channel, message, timestamp)
        """
        try:
            self._ensure_connected()
            request = redis_service_pb2.SubscribeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                channels=channels
            )

            for message in self.stub.Subscribe(request):
                if callback:
                    callback(message.channel, message.message, message.timestamp)

        except Exception as e:
            self.handle_error(e, "Subscribe to channels")

    def unsubscribe(self, channels: List[str]) -> bool:
        """Unsubscribe from channels"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.UnsubscribeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                channels=channels
            )

            response = self.stub.Unsubscribe(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Unsubscribe from channels")
            return False

    # ============================================
    # Batch Operations
    # ============================================

    def execute_batch(self, commands: List[Dict[str, any]]) -> Optional[Dict]:
        """
        Execute batch commands
        
        Args:
            commands: List of command dicts with keys: operation, key, value, expiration
        """
        try:
            self._ensure_connected()
            
            batch_commands = []
            for cmd in commands:
                from google.protobuf.duration_pb2 import Duration
                
                expiration = None
                if 'expiration' in cmd and cmd['expiration']:
                    expiration = Duration()
                    expiration.seconds = cmd['expiration']
                
                batch_cmd = redis_service_pb2.BatchCommand(
                    operation=cmd.get('operation', ''),
                    key=cmd.get('key', ''),
                    value=cmd.get('value', ''),
                    expiration=expiration
                )
                batch_commands.append(batch_cmd)
            
            request = redis_service_pb2.RedisBatchRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                commands=batch_commands
            )

            response = self.stub.ExecuteBatch(request)

            if response.success:
                return {
                    'success': True,
                    'executed_count': response.executed_count,
                    'errors': list(response.errors)
                }
            else:
                return {
                    'success': False,
                    'executed_count': response.executed_count,
                    'errors': list(response.errors)
                }

        except Exception as e:
            self.handle_error(e, "Execute batch")
            return None

    # ============================================
    # Session Management
    # ============================================

    def create_session(self, data: Dict[str, str], ttl_seconds: int = 3600) -> Optional[str]:
        """Create session"""
        try:
            self._ensure_connected()
            
            from google.protobuf.duration_pb2 import Duration
            ttl = Duration()
            ttl.seconds = ttl_seconds
            
            request = redis_service_pb2.CreateSessionRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                data=data,
                ttl=ttl
            )

            response = self.stub.CreateSession(request)

            return response.session_id

        except Exception as e:
            return self.handle_error(e, "Create session")

    def get_session(self, session_id: str) -> Optional[Dict]:
        """Get session"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.GetSessionRequest(
                user_id=self.user_id,
                session_id=session_id
            )

            response = self.stub.GetSession(request)

            if response.found:
                return self._proto_map_to_dict(response.session.data)
            else:
                return None

        except Exception as e:
            return self.handle_error(e, "Get session")

    def update_session(self, session_id: str, data: Dict[str, str], extend_ttl: bool = True) -> bool:
        """Update session"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.UpdateSessionRequest(
                user_id=self.user_id,
                session_id=session_id,
                data=data,
                extend_ttl=extend_ttl
            )

            response = self.stub.UpdateSession(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Update session")
            return False

    def delete_session(self, session_id: str) -> bool:
        """Delete session"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.DeleteSessionRequest(
                user_id=self.user_id,
                session_id=session_id
            )

            response = self.stub.DeleteSession(request)

            if response.success:
                return True
            else:
                return False

        except Exception as e:
            self.handle_error(e, "Delete session")
            return False

    def list_sessions(self) -> List[Dict]:
        """List all sessions"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.ListSessionsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = self.stub.ListSessions(request)

            sessions = []
            for session in response.sessions:
                sessions.append({
                    'session_id': session.session_id,
                    'user_id': session.user_id,
                    'data': dict(session.data),
                    'created_at': session.created_at,
                    'expires_at': session.expires_at
                })

            return sessions

        except Exception as e:
            return self.handle_error(e, "List sessions") or []

    # ============================================
    # Monitoring Operations
    # ============================================

    def get_statistics(self) -> Optional[Dict]:
        """Get Redis statistics"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.GetStatisticsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = self.stub.GetStatistics(request)

            stats = {
                'total_keys': response.total_keys,
                'memory_used_bytes': response.memory_used_bytes,
                'commands_processed': response.commands_processed,
                'connections_received': response.connections_received,
                'hit_rate': response.hit_rate,
                'key_type_distribution': self._proto_map_to_dict(response.key_type_distribution)
            }

            return stats

        except Exception as e:
            return self.handle_error(e, "Get statistics")

    def get_key_info(self, key: str) -> Optional[Dict]:
        """Get key information"""
        try:
            self._ensure_connected()
            request = redis_service_pb2.GetKeyInfoRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = self.stub.GetKeyInfo(request)

            if response.exists:
                info = {
                    'exists': True,
                    'type': response.type,
                    'ttl_seconds': response.ttl_seconds,
                    'size_bytes': response.size_bytes,
                    'created_at': response.created_at,
                    'last_accessed': response.last_accessed
                }
                return info
            else:
                return {'exists': False}

        except Exception as e:
            return self.handle_error(e, "Get key info")


# Convenience usage example
if __name__ == '__main__':
    with RedisClient(host='localhost', port=50055, user_id='test_user') as client:
        # Health check
        client.health_check()

        # Basic operations
        client.set('user:1:name', 'John Doe')
        name = client.get('user:1:name')

        # With TTL
        client.set_with_ttl('session:abc123', 'user_data', ttl_seconds=3600)
        ttl = client.ttl('session:abc123')

        # Batch operations
        client.mset({
            'user:2:name': 'Jane Smith',
            'user:2:email': 'jane@example.com',
            'user:2:age': '25'
        })

        users = client.mget(['user:2:name', 'user:2:email', 'user:2:age'])

        # Counter
        counter = client.incr('page:views')

        # List operations
        client.lpush('logs', ['log1', 'log2', 'log3'])
        logs = client.lrange('logs', 0, -1)

        # Hash operations
        client.hset('user:3', 'name', 'Bob Wilson')
        client.hset('user:3', 'email', 'bob@example.com')
        user_data = client.hgetall('user:3')
